import os
import sys
qstring = 'Does each file match(Y/N): If not rename files to match and rerun'


def run(fn, ldir, wdir):
    """run the setup"""
    logs = os.listdir(ldir)
    wavs = os.listdir(wdir)
    wavs = [w for w in wavs if w[-1] == 'v']
    for w in range(len(wavs)):
        print(wavs[w], logs[w])
    if input(qstring) == 'Y':
        labels = []
        for l in logs:
            with open(os.path.join(ldir, l), 'r') as lf:
                rtxt = lf.read().splitlines()
            labels.append(rtxt[5].split('\t')[0])

        tt = ''
        for r in range(len(wavs)):
            lfn = os.path.join(ldir, logs[r])
            wfn = os.path.join(wdir, wavs[r])
            line = [labels[r], wfn, lfn, '0.012', '0.2']
            tt += '\t'.join(line) + '\n'
        with open(fn, 'w') as tf:
            tf.write(tt)

if __name__ == '__main__':
    run(*sys.argv[1:])



# coding: utf-8
import numpy as np
with open('pilot01-FoodAddiction3.log', 'r') as lf:
    lines = lf.read().splitlines()
    
header = lines[3].split('\t')
data = [l.split('\t') for l in lines[5:]]
codecol = header.index('Code')
timecol = header.index('Time')
catcol = header.index('Category(str)')
for t in data:
    if t[codecol] == '199':
        firstpulse = int(t[timecol])
        break
    
categories = {}
categories['fixations'] = []
categories['gaps'] = []
categories['pulses'] = []
inblock = False
for e in data:
    e[timecol] = int(e[timecol])

for t in data:
    if t[codecol] == '199':
        categories['pulses'].append(t[timecol] - firstpulse)
    elif t[codecol] == 'fix':
        categories['fixations'].append(t[timecol] - firstpulse)
        inblock = False
    elif t[codecol] == 'gap':
        categories['gaps'].append(t[timecol] - firstpulse)
        inblock = False
    elif t[catcol]:
        if not inblock:
            if t[catcol] in categories:
                categories[t[catcol]].append(t[timecol] - firstpulse)
            else:
                categories[t[catcol]] = [t[timecol] - firstpulse]
            inblock = True

np.set_printoptions(precision=2, suppress=True, linewidth=20000)
with open('blocks.m', 'w') as bf:
    for a in categories:
        print(a, np.array(categories[a]))
        bf.write(a + ' = ' + str(np.around(np.array(categories[a])/10000.0, 2)) + '\n')



# coding: utf-8
from sndan import scla, ExtractError
import sys


def runit(orig, rtext, settings, factors, measures):
    """Run the whole procedure with certain settings"""
    for t in orig:
        print(t[0])
        settings['mdur'] = float(t[3])
        settings['thresh'] = float(t[4])
        try:
            res = scla(t[0], t[1], t[2], **settings)
        except ValueError as e:
            print(e.message)
            continue
        except ExtractError as e:
            print(e.message)
            continue
        print('Successfully extracted')

        results = [t[0], t[3], t[4]]
        for f in factors:
            for m in measures:
                results.append(str(round(res[f][m], 1)))

        rtext += '\t'.join(results) + '\n'
    return rtext


def runall(fn, ofn):
    """Use runit on all lines extracted from fn and write to ofn"""
    with open(fn, 'r') as tf:
        tests = [t.split('\t') for t in tf.read().splitlines()]

    settings = {'mdur': 0.012, 'thresh': 0.2}  # Defaults
    factors = ['Lower Bound', 'Upper Bound', 'Port Time Diffs',
               'Snd Time Diffs', 'Port Code Lengths']
    measures = ['mean', 'min', 'max', 'stddev']

    rtext = 'ID\tmdur\tthresh\t'
    rtext += '\t'.join([f+'_'+m for f in factors for m in measures]) + '\n'
    rtext = runit(tests, rtext, settings, factors, measures)
    with open(ofn, 'w') as rf:
        rf.write(rtext)


if __name__ == '__main__':
    runall(*sys.argv[1:])



# coding: utf-8
from __future__ import division
from scipy.io import wavfile
import matplotlib.pyplot as plt
import numpy as np
from ..logfile import load


class ExtractError(Exception):
    """Raised when an extraction goes wrong"""
    def __init__(self, logData, portData, sndData):
        super(ExtractError, self).__init__()
        self.logData, self.portData, self.sndData = logData, portData, sndData
        self.message = 'Extraction error, detected code lengths do not match'

    def __str__(self):
        """Represent this error as a string"""
        elements = [self.message, len(self.logData), len(self.portData), len(self.sndData)]
        return '{}\nLengths: Log({}), Port({}), Snds({})'.format(*elements)


class ConvertError(Exception):
    """Raised when a log file number was not converted successfully"""
    def __init__(self, udat):
        super(ConvertError, self).__init__()
        self.udat = udat


def stdStats(datasets):
    """Run the full gamut of standard stats on each dataset"""
    stats = {}
    for d in datasets:
        stats[d] = {}
        data = datasets[d]
        stats[d]['mean'] = np.mean(data)
        stats[d]['min'] = min(data)
        stats[d]['max'] = max(data)
        stats[d]['stddev'] = np.std(data)
        stats[d]['rawdata'] = data
    return stats


def extract_channel_events(channel, maxdur=0.012, thresh=0.2, samplerate=44100):
    '''From a single channel, extract events that reach *thresh*old and last
    for *maxdur*'''
    events = []
    lastevt = -20000
    for index, value in enumerate(channel):
        if index - lastevt > maxdur*samplerate:
            if abs(value) > thresh:
                events.append(index/samplerate)
                lastevt = index
    return events


def extract_sound_events(soundfile, schannel=1, maxdur=0.012, thresh=0.2, plot=True, runid='scla'):
    fs, sdata = wavfile.read(soundfile)
    # Grab channels
    port = sdata.T[1-schannel]
    snd = sdata.T[schannel]
    # Normalise channels
    port = port/max(port)
    snd = snd/max(snd)
    port_events = extract_channel_events(port, maxdur=maxdur, thresh=thresh, samplerate=fs)
    snd_events = extract_channel_events(snd, maxdur=maxdur, thresh=thresh, samplerate=fs)
    if plot:
        plt.plot(snd[int(snd_events[0]*fs)-100:int((snd_events[0]+maxdur)*fs)])
        plt.savefig(runid+'_firstsnd.png')
        plt.close()
    return fs, port_events, snd_events, port


def scla(soundfile=None, logfile=None, **kwargs):
    """Implements similar logic to Neurobehavioural Systems SCLA program"""
    log = load(logfile)

    fs, pcodes, snds, port = extract_sound_events(soundfile, **kwargs)
    if (len(log.events) != len(pcodes)) or (len(pcodes) != len(snds)):
        raise ExtractError(log.events, pcodes, snds)

    datasets = {}
    datasets['Lower Bound'] = []
    datasets['Upper Bound'] = []
    for evt in range(len(snds)):
        datasets['Lower Bound'].append(snds[evt] - pcodes[evt])
        datasets['Upper Bound'].append(snds[evt] - pcodes[evt] +
                                       float(log.events[evt].data['Uncertainty (Time)'])*0.0001)  # Uncertainty in seconds
    td, pl = timing(port, pcodes, snds, fs, **kwargs)
    datasets['Port Time Diffs'] = td['pcodes']
    datasets['Snd Time Diffs'] = td['snds']
    datasets['Port Code Lengths'] = pl
#    import pdb; pdb.set_trace()
    return stdStats(datasets)


def timing(port, pcodes, snds, fs, maxdur=0, thresh=0, **kwargs):
    """extract extra info about port duration and time between stimuli"""
    timediffs = {'pcodes': [], 'snds': []}
    portlengths = []
    for p in range(1, len(pcodes)):
        timediffs['pcodes'].append(pcodes[p] - pcodes[p-1])
    for s in range(1, len(snds)):
        timediffs['snds'].append(snds[s] - snds[s-1])
    for p in pcodes:
        span = range(int(p*fs) + 1, int((p+maxdur*1000)*fs))
        for pt in span:
            if port[pt] < thresh:
                portlengths.append(pt/fs - p)
                break
    return timediffs, portlengths



from setuptools import setup, find_packages
import codecs
import os
import subprocess
import sys

__version__ = '0.0.6'

base_dir = os.path.abspath(os.path.dirname(__file__))


def genRST():
    pandoc_call = ['pandoc', '--from=markdown', '--to=rst', 'README.md']
    try:
        output = subprocess.run(pandoc_call, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if output.returncode:
            print(output.stderr)
            sys.exit()
        output = output.stdout
    except AttributeError:
        try:
            output = subprocess.check_output(pandoc_call)
        except subprocess.CalledProcessError:
            sys.exit()
    return output.decode()


# get the dependencies and installs
with codecs.open(os.path.join(base_dir, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if 'git+' not in x]

setup(
    name='prespy',
    version=__version__,
    description='Package for working with the Neurobehavioural Systems' +
                ' Presentation logfiles within python',
    long_description=genRST(),
    url='https://github.com/gjcooper/prespy',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
    ],
    author='Gavin Cooper',
    author_email='gjcooper@gmail.com',
    install_requires=install_requires,
    dependency_links=dependency_links,
    license='GPL v2.0',
    packages=find_packages(exclude=['docs', 'tests*']),
    include_package_data=True,
    keywords=['Presentation', 'NBS', 'logfile'],
    entry_points={
        'console_scripts': ['pres-scla=prespy.__main__:scla'],
    }
)
